<?php
/**
 * Nic_Device_SNMP:: Nic_Device implementation to retrieve data from
 * an SNMP service.
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @since   Nic 0.2
 * @package Nic
 */
class Nic_Device_SNMP extends Nic_Device {

    /**
     * Render the current details of the device in an HTML table
     *
     * @param string $templagePath  Path to the templates to use.
     * @param string $deviceid      ID of the device, if set is use to create links
     */
    function showDetails($templatePath, $device_id)
    {
        require $templatePath . '/header.inc';

        $details = $this->retrieveStoreableData();

        foreach ($details as $description => $values) {
            $value = $values['value'];
            require $templatePath . '/row.inc';
        }

        foreach ($this->_graphs as $graph_id => $graph) {
            require $templatePath . '/row-graph.inc';
        }

        require $templatePath . '/footer.inc';
    }

    /**
     * Retrieve the current details of the device.
     *
     * @return    array      KEY: description of detail VALUE: value of that details
     */
    function retrieveDetails()
    {
        $details = array();
        if (!extension_loaded('snmp')) {
            return $details;
        }

        foreach ($this->oids as $oid) {
            $details[$oid['name']] = array();
            $response = @snmpget($this->params['hostspec'] . ':' . $this->params['port'], $this->params['community'], $oid['oid'], $this->params['timeout'], 2);
            $details[$oid['name']]['value'] = $this->_cleanResponse($response);

            if (!isset($oid['data_type'])) {
                $details[$oid['name']]['data_type'] = NIC_DATA_COUNTER;
            } else {
                $details[$oid['name']]['data_type'] = $oid['data_type'];
            }
        }

        return $details;
    }

    /**
     * Retrieve the current details of the device that can be stored
     * in a storage backend.
     *
     * @return    array      KEY: description of detail VALUE: value of that details
     *
     */
    function retrieveStoreableData()
    {
        $details = array();
        if (!extension_loaded('snmp')) {
            return $details;
        }

        foreach ($this->oids as $oid) {
            $storeable = true;
            if (isset($oid['storeable'])) {
                $storeable = $oid['storeable'];
            }

            if ($storeable) {
                $details[$oid['name']] = array();
                $response = @snmpget($this->params['hostspec'] . ':' . $this->params['port'], $this->params['community'], $oid['oid'], $this->params['timeout'], 2);
                $details[$oid['name']]['value'] = $this->_cleanResponse($response);

                if (!isset($oid['data_type'])) {
                    $details[$oid['name']]['data_type'] = NIC_DATA_COUNTER;
                } else {
                    $details[$oid['name']]['data_type'] = $oid['data_type'];
                }
            }
        }

        return $details;
    }

    /**
     * Clean the response from an SNMP get as differenct versions
     * return different things.
     */
    function _cleanResponse($response)
    {
        $pos = strpos($response, 'Counter32:');
        if ($pos !== false) {
            return trim(substr($response, $pos + 10));
        }
        $pos = strpos($response, 'Gauge32');
        if ($pos !== false) {
            return trim(substr($response, $pos + 8));
        }
        $pos = strpos($response, 'Timeticks:');
        if ($pos !== false) {
            return trim(substr($response, $pos + 10));
        }
        $pos = strpos($response, '=');
        if ($pos !== false) {
            return trim(substr($response, $pos + 1));
        }
        return $response;
    }

}
