<?php
/**
 * Nic_Storage_RRD:: RRDTool storage backends for Network.
 *
 * $Horde: nic/lib/Storage/rrdtool.php
 *
 * @author  Mike Bydalek <mike@distance.net>
 * @since   Network 0.1
 * @package Nic
 */
class Nic_Storage_RRD extends Nic_Storage {

    /** Hash containing connection parameters. */
    var $_params = array();

    var $_rrdtool;

    var $_rrdfile;

    /**
     * Constructs a new RRDTool storage object.
     *
     * @param string $device    Device ID to store / retrive values for.
     * @param array  $params    A hash containing connection parameters.
     */
    function Nic_Storage_RRD($params)
    {
        $this->_params = $params;
    }

    /**
     * Add a new data value to the storage system.
     *
     * @param string    $rrdfile    The name of the RRD file.
     * @param array     $values     Array containing values for RRD sources.
     *
     * @return mixed    (boolean) true on success, PEAR_Error on failure.
     */
    function add($rrdfile, $values)
    {
        /* Make sure we have a valid rrdtool setup. */
        $result = $this->_validate($rrdfile);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $cmd = $this->_rrdtool . ' update ' . $this->_rrdfile . ' N';
        foreach ($values as $value) {
            $cmd .= ':' . $value;
        }

        // Run the command and also grab the output.
        $handle = popen($cmd . ' 2>&1', 'r');
        $result = fread($handle, 2096);
        pclose($handle);

        // Return an error if RRDTool error'd out.
        if (eregi("ERROR", $result)) {
            return PEAR::raiseError('RRDTool returned: ' . $result);
        }

        return true;
    }

    /**
     * Attempts to validate a proper RRDTool configuration.
     *
     * @param string    $file   The name of the .rrd file to check.
     *
     * @return boolean    True on success, PEAR_Error on failure.
     */
    function _validate($file)
    {
        global $conf;

        // Set some misc. variables that will be referenced.
        $this->_rrdtool = $conf['rrdtool']['binary'];
        $this->_rrdfile = $this->getFileName($file);

        // Check if binary exists.
        if (!file_exists($this->_rrdtool)) {
            return PEAR::raiseError('Unable to find RRDTool binary: ' . $this->_rrdtool);
        }

        // Check if .rrd file exists.
        if (!file_exists($this->_rrdfile)) {
            return PEAR::raiseError('Unable to find RRD File: ' . $this->_rrdfile);
        }

        return true;
    }

    function getFileName($file)
    {
        global $conf;

        return $conf['rrdtool']['storage'] . '/' . $file . '.rrd';
    }

}
